#if !defined( __CONTEGGIO_H )
#define __CONTEGGIO_H


typedef std::vector<std::vector<int> >          tensor2;
typedef std::vector<std::vector<std::vector<int> > > tensor3;

//norm |a,b| of coprime integers
int norma(int a, int b);

//matrix
class matrice {
 public:
  int val[2][2];
  matrice();
  matrice(int a, int b, int c, int d);
  int* operator[] (int i) {return val[i];}
  const int* operator[] (int i) const {return val[i];}
};

//Seifert manifold
class Seifert_manifold {
 public:
  int chi_f;
  tensor2 param;
  int b;
  Seifert_manifold():chi_f(0), param(), b(0) {}
  Seifert_manifold(int c, const tensor2& p, int b=0):chi_f(c), param(p), b(b) {}
  void inverti();
  float e() const;
  float chi() const;
  int geometria() const;
  int norma(int i) const {return ::norma(param[i][0], param[i][1]);}
  int quante_fibre() const {return param.size();}
  void scrivi(const int& quanti) const;
  int d3() const;
  BOOL forma_standard() const;
  BOOL cij() const;
  BOOL from_B5() const;
};

//graph manifold
class graph_manifold {
 public:
  std::vector<Seifert_manifold> blocchi;  //the parameter b is considered to be always zero.
  tensor2 collegamenti;
  std::vector<matrice> A;
  graph_manifold():blocchi(), collegamenti(), A() {}
  graph_manifold(const std::vector<Seifert_manifold>& b, const tensor2& c,
		 const std::vector<matrice>& A):blocchi(b), collegamenti(c), A(A) {}
  void inverti(int i);
  void ruota();
  void set_forma_standard();
	BOOL orientable() const;
  BOOL line() const;
	BOOL self_assembling() const;
	BOOL ruotabile() const;
  BOOL forma_standard() const;
  BOOL is_JSJ() const;
  BOOL is_Sol() const;
  BOOL is_Seifert() const;
  BOOL due_fibrazioni(int i) const;
	int quanti_collegamenti(int num_blocco) const;
	void cambia_sezione(int num_blocco, int param);
  void scrivi(int quanti) const;
  int param_somma_norme() const;
  int d3() const;
  int d3_senza_matrici() const;
};

BOOL operator<(const Seifert_manifold& M1, const Seifert_manifold& M2);
BOOL operator<(const graph_manifold& M1, const graph_manifold& M2);
BOOL operator==(const graph_manifold& M1, const graph_manifold& M2);

matrice operator-(const matrice& A);
matrice operator*(const matrice& A, const matrice& B);
BOOL operator==(const matrice& A, const matrice& B);
int det(const matrice& A);

#endif
